/* This file is a part of the SimpleScalar tool suite written by
 * Todd M. Austin, University of Wisconsin - Madison, Computer Sciences
 * Department as a part of the Multiscalar Research Project.
 *  
 * The SimpleScalar x86 port was written by Steve Bennett.
 * 
 * The tool suite is currently maintained by Doug Burger.
 * 
 * Copyright (C) 1994, 1995, 1996 by Todd M. Austin
 *
 * This source file is distributed "as is" in the hope that it will be
 * useful.  The tool set comes with no warranty, and no author or
 * distributor accepts any responsibility for the consequences of its
 * use. 
 * 
 * Everyone is granted permission to copy, modify and redistribute
 * this tool set under the following conditions:
 * 
 *    This source code is distributed for non-commercial use only. 
 *    Please contact the maintainer for restrictions applying to 
 *    commercial use.
 *
 *    Permission is granted to anyone to make or distribute copies
 *    of this source code, either as received or modified, in any
 *    medium, provided that all copyright notices, permission and
 *    nonwarranty notices are preserved, and that the distributor
 *    grants the recipient permission for further redistribution as
 *    permitted by this document.
 *
 *    Permission is granted to distribute this file in compiled
 *    or executable form under the same conditions that apply for
 *    source code, provided that either:
 *
 *    A. it is accompanied by the corresponding machine-readable
 *       source code,
 *    B. it is accompanied by a written offer, with no time limit,
 *       to give anyone a machine-readable copy of the corresponding
 *       source code in return for reimbursement of the cost of
 *       distribution.  This written offer must permit verbatim
 *       duplication by anyone, or
 *    C. it is distributed by someone who received only the
 *       executable form, and is accompanied by a copy of the
 *       written offer of source code that they received concurrently.
 *
 * In other words, you are welcome to use, share and improve this
 * source file.  You are forbidden to forbid anyone else to use, share
 * and improve what you give them.
 *
 * INTERNET: dburger@cs.wisc.edu
 * US Mail:  1210 W. Dayton Street, Madison, WI 53706
 */
#ifndef ERRORROUTINES_H
#define ERRORROUTINES_H

#include "types.h"

#include "ix86_state.h"




#define error(fmt, args...)	\
    _error(__FILE__, __FUNCTION__, __LINE__, fmt, ## args)

extern void
_error(char *file, char *func, int line, char *fmt, ...)
__attribute__ ((noreturn));

/* register a function to be called when an error is detected */
extern void RegisterErrorFn(void (*efn)(FILE *stream, cpu_t *state));

#define fatal(fmt, args...)	\
    _fatal(__FILE__, __FUNCTION__, __LINE__, fmt, ## args)

extern void
_fatal(char *file, char *func, int line, char *fmt, ...)
__attribute__ ((noreturn));

#define panic(fmt, args...)	\
    _panic(__FILE__, __FUNCTION__, __LINE__, fmt, ## args)

extern void
_panic(char *file, char *func, int line, char *fmt, ...)
__attribute__ ((noreturn));

#define warn(fmt, args...)	\
    _warn(__FILE__, __FUNCTION__, __LINE__, fmt, ## args)

#define tripwarn(var, fmt, args...)	\
    do {				\
	if (!var) {			\
	    _warn(__FILE__, __FUNCTION__, __LINE__, fmt, ## args); \
	    var = TRUE;			\
	}				\
    } while (0)

extern void
_warn(char *file, char *func, int line, char *fmt, ...);

#ifndef NDEBUG
extern void
_my_assert(char *file, char *func, int line, char *ex)
__attribute__ ((noreturn));

#define assert(EX)						\
  (EX)?(0): _my_assert(__FILE__, __FUNCTION__, __LINE__, #EX)
#else
#define assert(EX)	(0)
#endif /* NDEBUG */



extern void Usage(int argc, char **argv);
extern void HandleSIGQUIT(long sigtype);
extern void HandleSIGILL(long sigtype);
extern void HandleSIGABRT(long sigtype);
extern void HandleSIGEMT(long sigtype);
extern void HandleSIGFPE(long sigtype);
extern void HandleSIGBUS(long sigtype);
extern void HandleSIGSEGV(long sigtype);
extern void HandleSIGSYS(long sigtype);
extern void HandleSIGLOST(long sigtype);

#endif
