/* This file is a part of the SimpleScalar tool suite written by
 * Todd M. Austin, University of Wisconsin - Madison, Computer Sciences
 * Department as a part of the Multiscalar Research Project.
 *  
 * The SimpleScalar x86 port was written by Steve Bennett.
 * 
 * The tool suite is currently maintained by Doug Burger.
 * 
 * Copyright (C) 1994, 1995, 1996 by Todd M. Austin
 *
 * This source file is distributed "as is" in the hope that it will be
 * useful.  The tool set comes with no warranty, and no author or
 * distributor accepts any responsibility for the consequences of its
 * use. 
 * 
 * Everyone is granted permission to copy, modify and redistribute
 * this tool set under the following conditions:
 * 
 *    This source code is distributed for non-commercial use only. 
 *    Please contact the maintainer for restrictions applying to 
 *    commercial use.
 *
 *    Permission is granted to anyone to make or distribute copies
 *    of this source code, either as received or modified, in any
 *    medium, provided that all copyright notices, permission and
 *    nonwarranty notices are preserved, and that the distributor
 *    grants the recipient permission for further redistribution as
 *    permitted by this document.
 *
 *    Permission is granted to distribute this file in compiled
 *    or executable form under the same conditions that apply for
 *    source code, provided that either:
 *
 *    A. it is accompanied by the corresponding machine-readable
 *       source code,
 *    B. it is accompanied by a written offer, with no time limit,
 *       to give anyone a machine-readable copy of the corresponding
 *       source code in return for reimbursement of the cost of
 *       distribution.  This written offer must permit verbatim
 *       duplication by anyone, or
 *    C. it is distributed by someone who received only the
 *       executable form, and is accompanied by a copy of the
 *       written offer of source code that they received concurrently.
 *
 * In other words, you are welcome to use, share and improve this
 * source file.  You are forbidden to forbid anyone else to use, share
 * and improve what you give them.
 *
 * INTERNET: dburger@cs.wisc.edu
 * US Mail:  1210 W. Dayton Street, Madison, WI 53706
 */

#ifndef TRANS_HEAD
#define TRANS_HEAD

#include <stdio.h>

#include "misc.h"
#include "ix86_state.h"

typedef struct {
	unsigned long             realPC;
	int               iSize;
	
	enum Opcode       op;
	Bit16u            opcodeByte;

	/* modr/m byte */
	int              mod;
	int              rm;
	int              rmreg;

	/* sib byte */
	int               ss;
	int               index;
	int               base;

	enum sizes        opSize;
	enum sizes        addSize;
	enum repeats      repeat;
	enum overrides    segOver;
	Boolean           fwait;
	
	Bit32u            displacement;
	Bit32u            immediate;
	Bit32u            offset;
	
	unsigned long             branchTarget;
	unsigned long             nextInst;
} iInst_t;

#define MAX_CODE_SIZE 10000

/* enum Opcode -> boolean indicating if a modrm byte is used by this opcode */
extern int modrmPresent[];

/* enum Opcode -> lenghth of immediate field */
extern char op2immediate[];

/* enum Opcode -> opcode operand format */
#define OP_FORMAT(OP)   (op2format[OP])
extern char *op2format[];

extern void InitDecoder(void);
extern void dumpOpcodeInfo(FILE *outstream);
extern void DisassembleInst(iInst_t inst, FILE *stream, Bit32u eip);
extern iInst_t getInst(unsigned long pc);

#endif
